/*!
 * \file date.cpp
 * Contains the implementation of the business::Date class.
 */

#include "date.h"

#include <stdlib.h>  // for atoi()
#include <string.h>  // for strcpy()
#include <ctime>     // for time manipulation functions

#include <cstring>
using std::strtok;   // for tokenizer functions, to parse standard strings

#include <sstream>
using std::stringstream;

#include <vector>
using std::vector;

#ifndef DATAACCESSLAYER_H_
#include "../data/dataaccesslayer.h"
#endif

/*!
 * \namespace business
 * 
 * Contains all the core classes of the application. Classes on this
 * namespace only depend on the STL libraries, to make them highly 
 * portable and independent from the client application.
 */
namespace business
{
    /*!
     * Default constructor. Creates an instance with today's date.
     */
    Date::Date()
    : Persistable    ()
    , _isValid       (false)
    , _isLeapYear    (false)
    , _year          (0)
    , _month         (0)
    , _day           (0)
    , _kind          (NONE)
    , _personId      (Persistable::DEFAULT_ID)
    , _weekday       (std::string(""))
    , _standardString(std::string(""))
    {
        // basic time manipulation taken from
        // http://www.informit.com/guides/content.asp?g=cplusplus&seqNum=65&rl=1
        std::time_t curr;
        std::tm local;
        time(&curr);
        local = *(std::localtime(&curr));
        
        // now build the proper instance
        _year = local.tm_year + 1900;
        _month = local.tm_mon + 1;
        _day = local.tm_mday;
        
        calculateStandardString();
        init();
    }
    
    /*!
     * Constructor. Creates a date with the specified parameters.
     * 
     * The values passed in parameter are checked against the
     * valid boundaries, and in case that they are not valid,
     * the "isValid()" method would return false.
     * 
     * \param year An integer value representing the year.
     * \param month An integer value representing the month.
     * \param day An integer value representing the day.
     */
    Date::Date(const int year, const int month, const int day)
    : Persistable    ()
    , _isValid       (false)
    , _isLeapYear    (false)
    , _year          (year)
    , _month         (month)
    , _day           (day)
    , _kind          (NONE)
    , _personId      (Persistable::DEFAULT_ID)
    , _weekday       (std::string(""))
    , _standardString(std::string(""))
    {
        calculateStandardString();
        init();
    }
    
    /*!
     * Constructor. Creates a date based in the string passed in
     * parameter. The parameter must follow the ISO 8601 standard.
     * 
     * The values passed in parameter are checked against the
     * valid boundaries, and in case that they are not valid,
     * the "isValid()" method would return false.
     * 
     * \param standardString A string of the form "YYYY-MM-DD"
     */
    Date::Date(const std::string& standardString)
    : Persistable()
    , _isValid       (false)
    , _isLeapYear    (false)
    , _year          (0)
    , _month         (0)
    , _day           (0)
    , _kind          (NONE)
    , _personId      (Persistable::DEFAULT_ID)
    , _weekday       (std::string(""))
    , _standardString(standardString)
    {
        parseStandardString();
        init();
    }
    
    /*!
     * Private constructor. Creates an instance of the class
     * that exists in the database. Only used by the "find()" factory
     * methods, to create instances using the data from the database. 
     * 
     * \param standardString A string with the ISO representation of the date.
     * \param personId The ID of the person that this instance belongs to.
     * \param kind The type of Date of this instance.
     * \param id The ID of the current instance.
     */
    Date::Date(const std::string& standardString, 
                const PersistableId personId,
                const DateKind kind,
                const PersistableId id)
    : Persistable (id)
    , _isValid       (false)
    , _isLeapYear    (false)
    , _year          (0)
    , _month         (0)
    , _day           (0)
    , _kind          (kind)
    , _personId      (personId)
    , _weekday       (std::string(""))
    , _standardString(standardString)
    {
        parseStandardString();
        init();
    }

    /*!
     * Copy constructor.
     * 
     * \param rhs The instance to copy from.
     */
    Date::Date(const Date& rhs)
    : Persistable(rhs)
    , _isValid       (rhs._isValid)
    , _isLeapYear    (rhs._isLeapYear)
    , _year          (rhs._year)
    , _month         (rhs._month)
    , _day           (rhs._day)
    , _kind          (rhs._kind)
    , _personId      (rhs._personId)
    , _weekday       (rhs._weekday)
    , _standardString(rhs._standardString)
    {
    }

    /*!
     * Virtual destructor.
     */
    Date::~Date()
    {
    }
    
    /*!
     * Called by the constructors to initialize and validate
     * the current instance.
     */
    void Date::init()
    {
        calculateLeapYear();
        // We need to know whether the year it's leap
        // before validating the current instance!
        validate();
        calculateWeekday();
    }

    /*!
     * Static factory method that returns the Date corresponding
     * to the ID passed as parameter.
     * 
     * \param dateId The ID of the date sought.
     * 
     * \return A Date instance.
     */
    Date* Date::findById(PersistableId dateId)
    {
        Date* date = NULL;
        data::DataAccessLayer& dal = data::DataAccessLayer::get();
        bool ok = dal.open();
        if (ok)
        {
            std::stringstream query;
            query << "SELECT id, person_id, value, kind";
            query << " FROM dates";
            query << " WHERE id = ";
            query << dateId;
            ok = dal.executeQuery(query.str());
            dal.close();
            if (ok)
            {
                const std::vector<std::string>& data = dal.getData();
                const size_t dataItems = data.size();

                if (4 == dataItems)
                {
                    const PersistableId id = atoi(data[0].c_str());
                    const PersistableId personId = atoi(data[1].c_str());
                    const std::string value(data[2]);
                    const int kind = atoi(data[3].c_str());
                    date = new Date(value, personId, (DateKind)kind, id);
                }
            }
        }
        return date;
    }

    /*!
     * Static factory method that returns the Dates belonging
     * to the Person whose ID is passed as parameter.
     * 
     * \param personId The ID of the owner.
     * 
     * \return A vector of Date instances.
     */
    Dates* Date::findByPersonId(PersistableId personId)
    {
        Dates* dates = new Dates;
        data::DataAccessLayer& dal = data::DataAccessLayer::get();
        bool ok = dal.open();
        if (ok)
        {
            std::stringstream query;
            query << "SELECT id, person_id, value, kind";
            query << " FROM dates";
            query << " WHERE person_id = ";
            query << personId;
            query << " ORDER BY value, kind DESC;";
            ok = dal.executeQuery(query.str());
            dal.close();
            if (ok)
            {
                const std::vector<std::string>& data = dal.getData();
                const size_t numberOfHeaders = dal.getTableHeaders().size();
                const size_t dataItems = data.size();

                for (size_t i = 0; i < dataItems; 
                                    i = i + numberOfHeaders)
                {
                    const PersistableId id = atoi(data[i].c_str());
                    const PersistableId personId = atoi(data[i + 1].c_str());
                    const std::string value(data[i + 2]);
                    const int kind = atoi(data[i + 3].c_str());
                    const Date date(value, personId, (DateKind)kind, id);
                    dates->push_back(date);
                }
            }
        }
        return dates;
    }

    /*!
     * Returns a textual description for the DateKind value
     * passed as parameter.
     * 
     * \param kind A DateKind value.
     * 
     * \return A string.
     */
    const std::string Date::getKindDescription(const DateKind kind)
    {
        std::string description;
        switch (kind)
        {
            case BIRTHDAY:
                description = "Birthday";
                break;
                
            case ANNIVERSARY:
                description = "Anniversary";
                break;
                
            default:
                description = "(n/a)";
                break;
        }
        return description;
    }

    /*!
     * Pure virtual method used by subclasses to store their
     * state in the database for the first time, using the
     * "INSERT" SQL statement. Subclasses must perform the insertion
     * and provide the ID that was given during the operation.
     * 
     * \return The ID of the current instance, as provided by SQLite.
     */
    PersistableId Date::insert()
    {
        PersistableId id = Persistable::DEFAULT_ID;
        if (_isValid && (_personId != Persistable::DEFAULT_ID))
        {
            data::DataAccessLayer& dal = data::DataAccessLayer::get();
            bool ok = dal.open();
            if (ok)
            {
                std::stringstream query;
                query << "INSERT INTO dates (person_id, value, kind)";
                query << " VALUES (";
                query << _personId;
                query << ", '";
                query << _standardString;
                query << "', ";
                query << (int)_kind;
                query << ");";
                ok = dal.executeQuery(query.str());
                dal.close();
                id = dal.getLastRowId();
            }
        }
        return id;
    }

    /*!
     * Pure virtual method used by subclasses to store their
     * state in the database after the first time, using the
     * "UPDATE" SQL statement.
     */
    void Date::update()
    {
        if (_isValid)
        {
            data::DataAccessLayer& dal = data::DataAccessLayer::get();
            bool ok = dal.open();
            if (ok)
            {
                std::stringstream query;
                query << "UPDATE dates SET value = '";
                query << _standardString;
                query << "', kind = ";
                query << (int)_kind;
                query << " WHERE id = ";
                query << getId();
                query << ";";
                ok = dal.executeQuery(query.str());
                dal.close();
            }
        }
    }

    /*!
     * Deletes from the database the Date specified by its ID.
     * 
     * \param id The ID of the instance to delete.
     */
    void Date::destroy(const PersistableId id)
    {
        data::DataAccessLayer& dal = data::DataAccessLayer::get();
        bool ok = dal.open();
        if (ok)
        {
            std::stringstream query;
            query << "DELETE FROM dates WHERE id = ";
            query << id;
            query << ";";
            ok = dal.executeQuery(query.str());
            dal.close();
        }
    }
    
    /*!
     * Returns the year of the current instance.
     * 
     * \return An integer.
     */
    const int Date::getYear() const
    {
        return _year;
    }
    
    /*!
     * Returns the month of the current instance.
     * 
     * \return An integer.
     */
    const int Date::getMonth() const
    {
        return _month;
    }

    /*!
     * Returns the textual name of the month of the current instance.
     * 
     * \return A string.
     */
    const std::string Date::getMonthName() const
    {
        std::string monthName;
        switch (_month)
        {
            case 1:
                monthName = "January";
                break;

            case 2:
                monthName = "February";
                break;

            case 3:
                monthName = "March";
                break;

            case 4:
                monthName = "April";
                break;

            case 5:
                monthName = "May";
                break;

            case 6:
                monthName = "June";
                break;

            case 7:
                monthName = "July";
                break;

            case 8:
                monthName = "August";
                break;

            case 9:
                monthName = "September";
                break;

            case 10:
                monthName = "October";
                break;

            case 11:
                monthName = "November";
                break;

            case 12:
                monthName = "December";
                break;

            default:
                monthName = "";
                break;
        }
        return monthName;
    }
    
    /*!
     * Returns the day of the current instance.
     * 
     * \return An integer.
     */
    const int Date::getDay() const
    {
        return _day;
    }
    
    /*!
     * Returns the weekday of the current instance.
     * 
     * \return A string.
     */
    const std::string& Date::getWeekday() const
    {
        return _weekday;
    }

    /*!
     * Returns the ID of the owner of this instance.
     * 
     * \return A PersistableId (long long).
     */
    const PersistableId Date::getPersonId() const
    {
        return _personId;
    }
    
    /*!
     * Returns the kind of the current instance.
     * 
     * \return A DateKind value.
     */
    const DateKind Date::getKind() const
    {
        return _kind;
    }
    
    /*!
     * Returns an ISO 8601 standard string, representing the 
     * current instance.
     * 
     * \return A string.
     */
    const std::string& Date::toStandardString() const
    {
        return _standardString;
    }
    
    /*!
     * Returns a boolean stating whether the year of the current
     * instance is leap or not.
     * 
     * \return A boolean.
     */
    const bool Date::isLeapYear() const
    {
        return _isLeapYear;
    }
    
    /*!
     * Returns a boolean stating whether the current instance
     * is valid or not.
     * 
     * \return A boolean.
     */
    const bool Date::isValid() const
    {
        return _isValid;
    }
    
    /*!
     * Sets the ID of the owner of the current instance.
     * This method works only when the ID of the current instance
     * is Persistable::DEFAULT_ID; in all other cases, the 
     * ID of the owner is non-mutable.
     * 
     * \param personId The PersistableId of the owner.
     */
    void Date::setPersonId(const PersistableId personId)
    {
        if (Persistable::DEFAULT_ID == _personId)
        {
            _personId = personId;
            setDirty();
        }
    }
    
    /*!
     * Sets the kind of the current instance.
     * 
     * \param kind A DateKind value. 
     */
    void Date::setKind(const DateKind kind)
    {
        _kind = kind;
        setDirty();
    }
    
    /*!
     * Assignment operator.
     * 
     * \param rhs The instance to copy from.
     * 
     * \return A reference to the current object.
     */
    Date& Date::operator=(const Date& rhs)
    {
        _year           = rhs._year;
        _month          = rhs._month;
        _day            = rhs._day;
        _kind           = rhs._kind;
        _personId       = rhs._personId;
        _isValid        = rhs._isValid;
        _isLeapYear     = rhs._isLeapYear;
        _weekday        = rhs._weekday;
        _standardString = rhs._standardString;
        return *this;
    }
    
    /*!
     * Equality operator.
     * 
     * \param rhs The instance to copy from.
     * 
     * \return A boolean.
     */
    const bool Date::operator==(const Date& rhs) const
    {
        bool result = (_year == rhs._year);
        result = result && (_month == rhs._month);
        result = result && (_day == rhs._day);
        return result;
    }

    /*!
     * Inequality operator.
     * 
     * \param rhs The instance to copy from.
     * 
     * \return A boolean.
     */
    const bool Date::operator!=(const Date& rhs) const
    {
        bool result = !(*this  == rhs);
        return result;
    }

    /*!
     * Less-than operator.
     * 
     * \param rhs The instance to copy from.
     * 
     * \return A boolean.
     */
    const bool Date::operator<(const Date& rhs) const
    {
        bool smaller = false;
        if (_year < rhs._year)
        {
            smaller = true;
        }
        else
        {
            if (_year > rhs._year)
            {
                smaller = false;
            }
            else
            {
                // Both years are equal: compare months
                if (_month < rhs._month)
                {
                    smaller = true;
                }
                else
                {
                    if (_month > rhs._month)
                    {
                        smaller = false;
                    }
                    else
                    {
                        // Both years and months are equal:
                        // compare days
                        if (_day < rhs._day)
                        {
                            smaller = true;
                        }
                        else
                        {
                            smaller = false;
                        }
                    }
                }
            }
        }
        return smaller;
    }
    
    /*!
     * Bigger-than operator.
     * 
     * \param rhs The instance to copy from.
     * 
     * \return A boolean.
     */
    const bool Date::operator>(const Date& rhs) const
    {
        bool result = !(*this < rhs) && !(*this == rhs);
        return result;
    }

    /*!
     * Less-or-equal-than operator.
     * 
     * \param rhs The instance to copy from.
     * 
     * \return A boolean.
     */
    const bool Date::operator<=(const Date& rhs) const
    {
        bool result = (*this < rhs) || (*this == rhs);
        return result;
    }
    
    /*!
     * Bigger-or-equal-than operator.
     * 
     * \param rhs The instance to copy from.
     * 
     * \return A boolean.
     */
    const bool Date::operator>=(const Date& rhs) const
    {
        bool result = (*this > rhs) || (*this == rhs);
        return result;
    }
    
    /*!
     * Validates the current instance (boundaries, leap years, etc)
     */
    void Date::validate()
    {
        bool validYearRange = (_year >= 1752) && (_year <= 2199);
        bool validMonthRange = (_month >= 1) && (_month <= 12);
        
        // If validMonthRange = false, do not calculate validateDay()
        // since there could be an "index out of bounds" exception
        const bool validDay = validMonthRange && validateDay();

        _isValid = validYearRange && validMonthRange && validDay;
    }

    /*!
     * Validates the day of the current instance.
     * 
     * \return A boolean; true if the day is valid, false otherwise.
     */
    const bool Date::validateDay()
    {
        // Adapted from Deitel's "C++ How To Program, Fifth Edition"
        // ISBN 0-13-185757-6, page 537
        static const int daysPerMonth[13] = 
            {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
        
        bool result = false;
        if (_day > 0 && _day <= daysPerMonth[_month])
        {
            result = true;
        }
        
        if (_day == 29 && _month == 2 && _isLeapYear)
        {
            result = true;
        }
        
        return result;
    }
   
    /*!
     * Verifies if the year of the current instance is leap or not.
     */
    void Date::calculateLeapYear()
    {
        if ((_year % 4) == 0)
        {
            _isLeapYear = true;
        }
        if ((_year % 100) == 0)
        {
            // Year 1900 was not leap...
            _isLeapYear = false;
            
            if ((_year % 400) == 0)
            {
                // Year 2000 was leap...
                _isLeapYear = true;
            }
        }
    }
    
    /*!
     * Calculates the weekday of the current string, using the
     * Date::getCenturyCode() and Date::getMonthCode() methods.
     */
    void Date::calculateWeekday()
    {
        // Algorithm adapted from 
        // http://en.wikipedia.org/wiki/Calculating_the_day_of_the_week
        
        int centuryCode = getCenturyCode();
        int lastTwoDigits = _year - (_year / 100) * 100;
        int division = lastTwoDigits / 4;
        int monthCode = getMonthCode();
        int sum = centuryCode + lastTwoDigits + division + monthCode + _day;
        int dayCode = sum % 7;
        
        switch (dayCode)
        {
            case 0:
                _weekday = "Sunday";
                break;
                
            case 1:
                _weekday = "Monday";
                break;
                
            case 2:
                _weekday = "Tuesday";
                break;
                
            case 3:
                _weekday = "Wednesday";
                break;
                
            case 4:
                _weekday = "Thursday";
                break;
                
            case 5:
                _weekday = "Friday";
                break;
                
            case 6:
                _weekday = "Saturday";
                break;
                
            default:
                break;
        }
    }
    
    
    /*!
     * Creates an ISO 8601 standard string representing the current
     * instance.
     */
    void Date::calculateStandardString()
    {
        // ISO 8601 standard ("YYYY-MM-DD") described here:
        // http://www.cl.cam.ac.uk/~mgk25/iso-time.html
        std::stringstream ss;
        ss << _year;
        ss << "-";

        if (_month < 10)
        {
            ss << "0";
        }
        ss << _month;
        ss << "-";

        if (_day < 10)
        {
            ss << "0";
        }
        ss << _day;
        
        _standardString = ss.str();    
    }

    /*!
     * Parses an ISO 8601 standard string to extract the 
     * year, month and day.
     */
    void Date::parseStandardString()
    {
        // Make a copy to preserve the original string
        int length = _standardString.length();
        char* s = new char[length + 1];
        strcpy(s, const_cast<char*>(_standardString.c_str()));
        char* tokenPointer = strtok(s, "-");

        // Don't continue if it's not valid
        _isValid = (tokenPointer != NULL);
        if (_isValid)
        {
            _year = atoi(tokenPointer);

            tokenPointer = strtok(NULL, "-");

            // Don't continue if it's not valid
            _isValid = (tokenPointer != NULL);
            if (_isValid)
            {
                _month = atoi(tokenPointer);

                tokenPointer = strtok(NULL, "-");

                // Don't continue if it's not valid
                _isValid = (tokenPointer != NULL);
                if (_isValid)
                {
                    _day = atoi(tokenPointer);
                }    
            }
        }
        delete[] s;
    }

    /*!
     * Used by Date::calculateWeekday().
     * 
     * \return An integer.
     */
    const int Date::getCenturyCode() const
    {
        /*
         * 1752-1799     4
         * 1800-1899     2
         * 1900-1999     0
         * 2000-2099     6
         * 2100-2199     4
         */
        int result = 0;
        if (_year >= 1752 && _year <= 1799)
        {
            result = 4;
        }
        if (_year >= 1800 && _year <= 1899)
        {
            result = 2;
        }
        if (_year >= 1900 && _year <= 1999)
        {
            result = 0;
        }
        if (_year >= 2000 && _year <= 2099)
        {
            result = 6;
        }
        if (_year >= 2100 && _year <= 2199)
        {
            result = 4;
        }
        return result;
    }
    
    /*!
     * Used by Date::calculateWeekday().
     * 
     * \return An integer.
     */
    const int Date::getMonthCode() const
    {
        /*
         * January      0 (in leap year 6)
         * February     3 (in leap year 2)
         * March        3     
         * April        6     
         * May          1     
         * June         4     
         * July         6     
         * August       2
         * September    5
         * October      0
         * November     3
         * December     5
         */
        int result = 0;
        switch (_month)
        {
            case 1:
                result = (_isLeapYear) ? 6 : 0;
                break;
                
            case 2:
                result = (_isLeapYear) ? 2 : 3;
                break;
                
            case 3:
                result = 3;
                break;
                
            case 4:
                result = 6;
                break;
                
            case 5:
                result = 1;
                break;
            
            case 6:
                result = 4;
                break;
                
            case 7:
                result = 6;
                break;
                
            case 8:
                result = 2;
                break;
                
            case 9:
                result = 5;
                break;
                
            case 10:
                result = 0;
                break;
                
            case 11:
                result = 3;
                break;
                
            case 12:
                result = 5;
                break;
                
            default:
                result = -1;
                break;
        }
        return result;
    }
}
