/*!
 * \file date.h
 * Contains the interface of the business::Date class.
 */

#ifndef DATE_H_
#define DATE_H_

#include <string>
using std::string;

#include <vector>
using std::vector;

#ifndef PERSISTABLE_H_
#include "persistable.h"
#endif

/*!
 * \namespace business
 * 
 * Contains all the core classes of the application. Classes on this
 * namespace only depend on the STL libraries, to make them highly 
 * portable and independent from the client application.
 */
namespace business
{
    // Forward declaration for the typedef below
    class Date;

    //! Represents a vector of Date instances; done for readability.
    typedef std::vector<Date> Dates;
    
    /*!
     * Represents different types of Date objects.
     * 
     * Even if this could have been done using subclasses, I have chosen
     * this approach to make things simpler; the codes of this enum are
     * used directly in the "dates" SQLite database table, to differenciate
     * them. Using subclasses would have meant to have some sort of
     * "reflection" or "introspection" capabilities, to ask every instance
     * about their concrete subclass.
     */
    enum DateKind
    {
        NONE        = -1,
        BIRTHDAY    = 0,
        ANNIVERSARY = 1
    };

    /*!
     * \class Date
     * 
     * Represents a particular day, in the range January 1st, 1752 to
     * December 31st, 2199. Upon creation, clients should verify that the 
     * instance is valid calling the "isValid()" method. If the instance
     * is not valid, it will not be saved into the database. 
     * 
     * This class does not hold Time-related information such as 
     * hours or minutes. 
     * 
     * This class is non-mutable; only the "kind" of it can be set.
     */
    class Date : public Persistable
    {
    public:

        //@{ \name Constructors and Destructor
            
        /*!
         * Default constructor. Creates an instance with today's date.
         */
    	Date();
        
        /*!
         * Constructor. Creates a date with the specified parameters.
         * 
         * The values passed in parameter are checked against the
         * valid boundaries, and in case that they are not valid,
         * the "isValid()" method would return false.
         * 
         * \param year An integer value representing the year.
         * \param month An integer value representing the month.
         * \param day An integer value representing the day.
         */
        Date(int year, int month, int day);
        
        /*!
         * Constructor. Creates a date based in the string passed in
         * parameter. The parameter must follow the ISO 8601 standard.
         * 
         * The values passed in parameter are checked against the
         * valid boundaries, and in case that they are not valid,
         * the "isValid()" method would return false.
         * 
         * \param standardString A string of the form "YYYY-MM-DD"
         */
        Date(const std::string& standardString);
        
        /*!
         * Virtual destructor.
         */
    	virtual ~Date();

        /*!
         * Copy constructor.
         * 
         * \param rhs The instance to copy from.
         */
        Date(const Date& rhs);

        //@}

        //@{ \name Operators

        /*!
         * Assignment operator.
         * 
         * \param rhs The instance to copy from.
         * 
         * \return A reference to the current object.
         */
        Date& operator=(const Date& rhs);

        /*!
         * Equality operator.
         * 
         * \param rhs The instance to copy from.
         * 
         * \return A boolean.
         */
        const bool operator==(const Date& rhs) const;

        /*!
         * Inequality operator.
         * 
         * \param rhs The instance to copy from.
         * 
         * \return A boolean.
         */
        const bool operator!=(const Date& rhs) const;

        /*!
         * Less-than operator.
         * 
         * \param rhs The instance to copy from.
         * 
         * \return A boolean.
         */
        const bool operator<(const Date& rhs) const;

        /*!
         * Less-or-equal-than operator.
         * 
         * \param rhs The instance to copy from.
         * 
         * \return A boolean.
         */
        const bool operator<=(const Date& rhs) const;

        /*!
         * Bigger-than operator.
         * 
         * \param rhs The instance to copy from.
         * 
         * \return A boolean.
         */
        const bool operator>(const Date& rhs) const;

        /*!
         * Bigger-or-equal-than operator.
         * 
         * \param rhs The instance to copy from.
         * 
         * \return A boolean.
         */
        const bool operator>=(const Date& rhs) const;

        //@}

        //@{ \name Static Methods

        /*!
        * Static factory method that returns the Date corresponding
        * to the ID passed as parameter.
        * 
        * \param dateId The ID of the date sought.
        * 
        * \return A Date instance.
        */
        static Date* findById(PersistableId dateId);

        /*!
         * Static factory method that returns the Dates belonging
         * to the Person whose ID is passed as parameter.
         * 
         * \param personId The ID of the owner.
         * 
         * \return A vector of Date instances.
         */
        static Dates* findByPersonId(const PersistableId personId);
        
        /*!
         * Deletes from the database the Date specified by its ID.
         * 
         * \param id The ID of the instance to delete.
         */
        static void destroy(const PersistableId id);
        
        /*!
         * Returns a textual description for the DateKind value
         * passed as parameter.
         * 
         * \param kind A DateKind value.
         * 
         * \return A string.
         */
        static const std::string getKindDescription(const DateKind kind);

        //@}

        //@{ \name Setters and Getters
        
        /*!
         * Returns a boolean stating whether the current instance
         * is valid or not.
         * 
         * \return A boolean.
         */
        const bool isValid() const;
        
        /*!
         * Returns the year of the current instance.
         * 
         * \return An integer.
         */
        const int getYear() const;

        /*!
         * Returns the month of the current instance.
         * 
         * \return An integer.
         */
        const int getMonth() const;

        /*!
         * Returns the textual name of the month of the current instance.
         * 
         * \return A string.
         */
        const std::string getMonthName() const;

        /*!
         * Returns the day of the current instance.
         * 
         * \return An integer.
         */
        const int getDay() const;
        
        /*!
         * Returns the ID of the owner of this instance.
         * 
         * \return A PersistableId (long long).
         */
        const PersistableId getPersonId() const;
        
        /*!
         * Returns the kind of the current instance.
         * 
         * \return A DateKind value.
         */
        const DateKind getKind() const;
        
        /*!
         * Returns the weekday of the current instance.
         * 
         * \return A string.
         */
        const std::string& getWeekday() const;
        
        /*!
         * Returns a boolean stating whether the year of the current
         * instance is leap or not.
         * 
         * \return A boolean.
         */
        const bool isLeapYear() const;
        
        /*!
         * Sets the ID of the owner of the current instance.
         * This method works only when the ID of the current instance
         * is Persistable::DEFAULT_ID; in all other cases, the 
         * ID of the owner is non-mutable.
         * 
         * \param personId The PersistableId of the owner.
         */
        void setPersonId(const PersistableId personId);
        
        /*!
         * Sets the kind of the current instance.
         * 
         * \param kind A DateKind value. 
         */
        void setKind(const DateKind kind);
        
        /*!
         * Returns an ISO 8601 standard string, representing the 
         * current instance.
         * 
         * \return A string.
         */
        const std::string& toStandardString() const;

        //@}
        
    protected:
    
        /*!
         * Pure virtual method used by subclasses to store their
         * state in the database for the first time, using the
         * "INSERT" SQL statement. Subclasses must perform the insertion
         * and provide the ID that was given during the operation.
         * 
         * \return The ID of the current instance, as provided by SQLite.
         */
        virtual PersistableId insert();

        /*!
         * Pure virtual method used by subclasses to store their
         * state in the database after the first time, using the
         * "UPDATE" SQL statement.
         */
        virtual void update();
        
    private:
    
        /*!
         * Private constructor. Creates an instance of the class
         * that exists in the database. Only used by the "find()" factory
         * methods, to create instances using the data from the database. 
         * 
         * \param standardString A string with the ISO representation of the date.
         * \param personId The ID of the person that this instance belongs to.
         * \param kind The type of Date of this instance.
         * \param id The ID of the current instance.
         */
        Date(const std::string& standardString, 
            const PersistableId personId,
            const DateKind kind,
            const PersistableId id);

    private:
    
        /*!
         * Called by the constructors to initialize and validate
         * the current instance.
         */
        void init();
        
        /*!
         * Validates the current instance (boundaries, leap years, etc)
         */
        void validate();
        
        /*!
         * Validates the day of the current instance.
         * 
         * \return A boolean; true if the day is valid, false otherwise.
         */
        const bool validateDay();
        
        /*!
         * Verifies if the year of the current instance is leap or not.
         */
        void calculateLeapYear();
        
        /*!
         * Calculates the weekday of the current string, using the
         * Date::getCenturyCode() and Date::getMonthCode() methods.
         */
        void calculateWeekday();
        
        /*!
         * Creates an ISO 8601 standard string representing the current
         * instance.
         */
        void calculateStandardString();
        
        /*!
         * Parses an ISO 8601 standard string to extract the 
         * year, month and day.
         */
        void parseStandardString();
        
        /*!
         * Used by Date::calculateWeekday().
         * 
         * \return An integer.
         */
        const int getCenturyCode() const;

        /*!
         * Used by Date::calculateWeekday().
         * 
         * \return An integer.
         */
        const int getMonthCode() const;
    
    private:
        
        //! Indicates whether the current instance is valid or not.
        bool _isValid;
        
        //! Indicates whether the current instance happens in a leap year.
        bool _isLeapYear;
        
        //! Stores the year value of the current instance.
        int _year;

        //! Stores the month value of the current instance.
        int _month;

        //! Stores the day value of the current instance.
        int _day;
        
        //! Indicates the kind of date of the current instance.
        DateKind _kind;
        
        //! Indicates the owner of the current instance.
        PersistableId _personId;
        
        //! Stores the weekday (Monday, Tuesday...) of the current instance.
        std::string _weekday;
        
        //! Stores an ISO standard string representation of the current instance.
        std::string _standardString;
    };
}

#endif /*DATE_H_*/
