/*!
 * \file persistable.cpp
 * Contains the implementation of the business::Persistable class.
 */

#include "persistable.h"

/*!
 * \namespace business
 * 
 * Contains all the core classes of the application. Classes on this
 * namespace only depend on the STL libraries, to make them highly 
 * portable and independent from the client application.
 */
namespace business
{
    /*!
     * Provides the default value given to
     * 
     * \li New instances (those that have not yet been saved in database
     * \li Instances that failed to be saved in the database.
     */
    const PersistableId Persistable::DEFAULT_ID = -1;

    /*!
     * Default constructor. Sets the ID to DEFAULT_ID. Used
     * to create instances that do not exist yet in the database.
     */
    Persistable::Persistable()
    : _id        (DEFAULT_ID)
    , _isNew     (true)
    , _isDirty   (true)
    {
    }

    /*!
     * Constructor. Used to create instances whose state is
     * already present in the database. This constructor, however,
     * does not check the database to see whether they exist there.
     */
    Persistable::Persistable(PersistableId id)
    : _id        (id)
    , _isNew     (false)
    , _isDirty   (false)
    {
    }

    /*!
     * Copy constructor.
     * 
     * \param rhs The instance to copy from ("right hand side").
     */
    Persistable::Persistable(const Persistable& rhs)
    : _id        (rhs._id)
    , _isNew     (rhs._isNew)
    , _isDirty   (rhs._isDirty)
    {
    }

    /*!
     * Virtual destructor.
     */
    Persistable::~Persistable()
    {
    }

    /*!
     * Assignment operator.
     * 
     * \param rhs The instance to copy from ("right hand side").
     */
    Persistable& Persistable::operator=(const Persistable& rhs)
    {
        if (this == &rhs)
        {
            return *this;
        }
        _id = rhs._id;
        _isDirty = rhs._isDirty;
        _isNew = rhs._isNew;
        return *this;
    }

    /*!
     * This method is the core of the Persistable family of
     * classes. Clients use it to write the current state of
     * the instance into the database.
     * 
     * This is a "template method", that provides the basic 
     * framework for subclasses, that must provide an implementation
     * to "insert()" and "update()". 
     */
    void Persistable::save()
    {
        if (_isNew)
        {
            _id = insert();
            if (_id != DEFAULT_ID)
            {
                _isNew = false;
                _isDirty = false;
            }
        }
        else
        {
            if (_isDirty)
            {
                update();
                _isDirty = false;
            }
        }
    }
    
    /*!
     * Returns a value indicating if the current instance has been
     * modified since it was retrieved from the database.
     * 
     * \return A boolean value.
     */
    const bool Persistable::isDirty() const
    {
        return _isDirty;
    }

    /*!
     * Returns a value indicating whether the current instance
     * exists in the database or not.
     * 
     * \return A boolean value.
     */
    const bool Persistable::isNew() const
    {
        return _isNew;
    }

    /*!
     * Returns the ID of the current instance.
     * 
     * \return A PersistableId (long long) value.
     */
    const PersistableId Persistable::getId() const
    {
        return _id;
    }
    
    /*!
     * Used by subclasses to change the "_isDirty" flag in the 
     * Persistable base class; subclasses must call this method in
     * every "setter" method.
     */
    void Persistable::setDirty()
    {
        _isDirty = true;
    }
}
