/*!
 * \file persistable.h
 * Contains the interface of the business::Persistable class.
 */

#ifndef PERSISTABLE_H_
#define PERSISTABLE_H_

/*!
 * \namespace business
 * 
 * Contains all the core classes of the application. Classes on this
 * namespace only depend on the STL libraries, to make them highly 
 * portable and independent from the client application.
 */
namespace business
{
    //! Represents the ID stored in the database for each Persistable instance.
    typedef long long PersistableId;

    /*!
     * \class Persistable
     * Represents the base class for objects that are meant to 
     * be stored in a relational database, using the Active Record 
     * pattern, as described by Martin Fowler:
     * http://www.martinfowler.com/eaaCatalog/activeRecord.html
     */
    class Persistable
    {
    public:

        /*!
         * Provides the default value given to
         * 
         * \li New instances (those that have not yet been saved in database
         * \li Instances that failed to be saved in the database.
         */
        static const PersistableId DEFAULT_ID;

        /*!
         * Default constructor. Sets the ID to DEFAULT_ID. Used
         * to create instances that do not exist yet in the database.
         */
        Persistable();
        
        /*!
         * Constructor. Used to create instances whose state is
         * already present in the database. This constructor, however,
         * does not check the database to see whether they exist there.
         */
        Persistable(PersistableId id);
        
        /*!
         * Copy constructor.
         * 
         * \param rhs The instance to copy from ("right hand side").
         */
        Persistable(const Persistable& rhs);
        
        /*!
         * Virtual destructor.
         */
    	virtual ~Persistable();
        
        /*!
         * Assignment operator.
         * 
         * \param rhs The instance to copy from ("right hand side").
         */
        Persistable& operator=(const Persistable& rhs);

        /*!
         * Returns a value indicating if the current instance has been
         * modified since it was retrieved from the database.
         * 
         * \return A boolean value.
         */
        const bool isDirty() const;
        
        /*!
         * Returns a value indicating whether the current instance
         * exists in the database or not.
         * 
         * \return A boolean value.
         */
        const bool isNew() const;
        
        /*!
         * Returns the ID of the current instance.
         * 
         * \return A PersistableId (long long) value.
         */
        const PersistableId getId() const;

        /*!
         * This method is the core of the Persistable family of
         * classes. Clients use it to write the current state of
         * the instance into the database.
         * 
         * This is a "template method", that provides the basic 
         * framework for subclasses, that must provide an implementation
         * to "insert()" and "update()". 
         */
        void save();

    protected:

        /*!
         * Pure virtual method used by subclasses to store their
         * state in the database for the first time, using the
         * "INSERT" SQL statement. Subclasses must perform the insertion
         * and provide the ID that was given during the operation.
         * 
         * \return The ID of the current instance, as provided by SQLite.
         */
        virtual PersistableId insert() = 0;
        
        /*!
         * Pure virtual method used by subclasses to store their
         * state in the database after the first time, using the
         * "UPDATE" SQL statement.
         */
        virtual void update() = 0;
        
        /*!
         * Used by subclasses to change the "_isDirty" flag in the 
         * Persistable base class; subclasses must call this method in
         * every "setter" method.
         */
        void setDirty();

    private:

        //! The ID of the current instance.
        PersistableId _id;
        
        //! A flag indicating whether this instance exists in database or not.
        bool _isNew;
        
        //! A flag indicating whether this instance has been modified or not.
        bool _isDirty;
    };
}

#endif /*PERSISTABLE_H_*/
