/*!
 * \file person.cpp
 * Contains the implementation of the business::Person class.
 */

#include "person.h"

#include <sstream>
using std::stringstream;

#ifndef DATAACCESSLAYER_H_
#include "../data/dataaccesslayer.h"
#endif

/*!
 * \namespace business
 *
 * Contains all the core classes of the application. Classes on this
 * namespace only depend on the STL libraries, to make them highly
 * portable and independent from the client application.
 */
namespace business
{
    /*!
     * Public constructor. Creates an instance of the class
     * that does not exist (yet) in the database.
     *
     * \param firstName A string with the first name.
     * \param lastName A string with the last name.
     */
    Person::Person(const std::string& firstName,
                    const std::string& lastName)
    : Persistable()
    , _firstName (firstName)
    , _lastName  (lastName)
    , _dates     (new Dates())
    {
    }

    /*!
     * Private constructor. Creates an instance of the class
     * that exists in the database. Only used by the "find()" factory
     * methods, to create instances using the data from the database.
     *
     * \param firstName A string with the first name.
     * \param lastName A string with the last name.
     * \param id The ID of the instance.
     */
    Person::Person(const std::string& firstName,
                    const std::string& lastName,
                    const PersistableId id)
    : Persistable(id)
    , _firstName (firstName)
    , _lastName  (lastName)
    , _dates     (new Dates())
    {
    }

    /*!
     * Copy constructor.
     *
     * \param rhs The instance to copy from.
     */
    Person::Person(const Person& rhs)
    : Persistable(rhs)
    , _firstName (rhs._firstName)
    , _lastName  (rhs._lastName)
    , _dates     (new Dates())
    {
    }

    /*!
     * Virtual destructor.
     */
    Person::~Person()
    {
        delete _dates;
    }

    /*!
     * Assignment operator.
     *
     * \param rhs The instance to copy from.
     *
     * \return A reference to the current object.
     */
    Person& Person::operator=(const Person& rhs)
    {
        if (this == &rhs)
        {
            return *this;
        }
        _firstName = rhs._firstName;
        _lastName  = rhs._lastName;
        _dates     = new Dates();
        return *this;
    }

    /*!
     * Static factory method that returns the instance of the
     * person class that corresponds to the ID passed in parameter.
     * Clients should assert or verify that the person has been
     * retrieved successfully before operating on it.
     *
     * \param id The ID of the person sought.
     *
     * \return A pointer to a Person instance, or NULL.
     */
    Person* Person::findById(const PersistableId id)
    {
        Person* person = NULL;
        data::DataAccessLayer& dal = data::DataAccessLayer::get();
        bool ok = dal.open();
        if (ok)
        {
            std::stringstream query;
            query << "SELECT id, first_name, last_name";
            query << " FROM people WHERE id = ";
            query << id;
            query << ";";
            ok = dal.executeQuery(query.str());
            dal.close();
            if (ok)
            {
                const std::vector<std::string>& data = dal.getData();
                const size_t dataItems = data.size();

                if (3 == dataItems)
                {
                    const PersistableId id = atoi(data[0].c_str());
                    const std::string firstName(data[1]);
                    const std::string lastName(data[2]);
                    person = new Person(firstName, lastName, id);
                }
            }
        }
        return person;
    }

    /*!
     * Static factory method that returns all the Person
     * instances that have some event at the specified date.
     *
     * \param date The date of which we want people with events.
     *
     * \return A vector of Person instances.
     */
    People* Person::findByDateEvent(const Date& date)
    {
        People* people = new People;
        data::DataAccessLayer& dal = data::DataAccessLayer::get();
        bool ok = dal.open();
        if (ok)
        {
            int month = date.getMonth();
            int day = date.getDay();
            std::stringstream query;
            query << "SELECT DISTINCT people.id, people.first_name, people.last_name";
            query << " FROM people";
            query << " INNER JOIN dates ON people.id = dates.person_id";
            query << " WHERE dates.value like '%-";
            if (month < 10)
            {
                query << "0";
            }
            query << month;
            query << "-";
            if (day < 10)
            {
                query << "0";
            }
            query << day;
            query << "'";
            query << " ORDER BY people.first_name, people.last_name;";
            ok = dal.executeQuery(query.str());
            dal.close();
            if (ok)
            {
                const std::vector<std::string>& data = dal.getData();
                const size_t numberOfHeaders = dal.getTableHeaders().size();
                const size_t dataItems = data.size();

                for (size_t i = 0; i < dataItems;
                                    i = i + numberOfHeaders)
                {
                    const PersistableId id = atoi(data[i].c_str());
                    const std::string firstName(data[i + 1]);
                    const std::string lastName(data[i + 2]);
                    const Person person(firstName, lastName, id);
                    people->push_back(person);
                }
            }
        }
        return people;
    }

    /*!
     * Static factory method that returns all the Person
     * instances that exist in the database.
     *
     * \return A vector of Person instances.
     */
    People* Person::findAll()
    {
        People* people = new People;
        data::DataAccessLayer& dal = data::DataAccessLayer::get();
        bool ok = dal.open();
        if (ok)
        {
            std::stringstream query;
            query << "SELECT id, first_name, last_name";
            query << " FROM people";
            query << " ORDER BY first_name, last_name;";
            ok = dal.executeQuery(query.str());
            dal.close();
            if (ok)
            {
                const std::vector<std::string>& data = dal.getData();
                const size_t numberOfHeaders = dal.getTableHeaders().size();
                const size_t dataItems = data.size();

                for (size_t i = 0; i < dataItems;
                                    i = i + numberOfHeaders)
                {
                    const PersistableId id = atoi(data[i].c_str());
                    const std::string firstName(data[i + 1]);
                    const std::string lastName(data[i + 2]);
                    const Person person(firstName, lastName, id);
                    people->push_back(person);
                }
            }
        }
        return people;
    }

    /*!
     * Pure virtual method used by subclasses to store their
     * state in the database for the first time, using the
     * "INSERT" SQL statement. Subclasses must perform the insertion
     * and provide the ID that was given during the operation.
     *
     * \return The ID of the current instance, as provided by SQLite.
     */
    PersistableId Person::insert()
    {
        data::DataAccessLayer& dal = data::DataAccessLayer::get();
        PersistableId id = Persistable::DEFAULT_ID;
        bool ok = dal.open();
        if (ok)
        {
            std::stringstream query;
            query << "INSERT INTO people (first_name, last_name)";
            query << " VALUES ('";
            query << _firstName;
            query << "', '";
            query << _lastName;
            query << "');";
            ok = dal.executeQuery(query.str());
            dal.close();
            id = dal.getLastRowId();

            // If there are dates added to this person, we must save them
            // now, because if their own personId is DEFAULT_ID, they won't be
            // saved into the database!
            Dates::iterator dateIterator;
            for(dateIterator = _dates->begin();
                dateIterator != _dates->end();
                dateIterator++)
            {
                Date currentDate = *dateIterator;
                currentDate.setPersonId(id);
                currentDate.save();
            }

        }
        return id;
    }

    /*!
     * Pure virtual method used by subclasses to store their
     * state in the database after the first time, using the
     * "UPDATE" SQL statement.
     */
    void Person::update()
    {
        data::DataAccessLayer& dal = data::DataAccessLayer::get();
        bool ok = dal.open();
        if (ok)
        {
            std::stringstream query;
            query << "UPDATE people SET first_name = '";
            query << _firstName;
            query << "', last_name = '";
            query << _lastName;
            query << "' WHERE id = ";
            query << getId();
            query << ";";
            ok = dal.executeQuery(query.str());
            dal.close();

            // If there are dates added to this person, we must save them
            Dates::iterator dateIterator;
            for(dateIterator = _dates->begin();
                dateIterator != _dates->end();
                dateIterator++)
            {
                Date currentDate = *dateIterator;
                currentDate.setPersonId(getId());
                currentDate.save();
            }
        }
    }

    /*!
     * Static method that performs the DELETE operation
     * on a single Person instance, specified by its ID.
     *
     * \param id The ID of the instance to delete.
     */
    void Person::destroy(const PersistableId id)
    {
        data::DataAccessLayer& dal = data::DataAccessLayer::get();
        bool ok = dal.open();
        if (ok)
        {
            std::stringstream query;
            query << "DELETE FROM people WHERE id = ";
            query << id;
            query << ";";
            ok = dal.executeQuery(query.str());
            dal.close();
        }
    }

    /*!
     * Static method that deletes all instances from the database.
     */
    void Person::destroyAll()
    {
        data::DataAccessLayer& dal = data::DataAccessLayer::get();
        bool ok = dal.open();
        if (ok)
        {
            std::stringstream query;
            ok = dal.executeQuery("DELETE FROM people;");
            dal.close();
        }
    }

    /*!
     * Sets the first name of the current instance.
     *
     * \param firstName A string.
     */
    void Person::setFirstName(const std::string& firstName)
    {
        _firstName = firstName;
        setDirty();
    }

    /*!
     * Sets the last name of the current instance.
     *
     * \param lastName A string.
     */
    void Person::setLastName(const std::string& lastName)
    {
        _lastName = lastName;
        setDirty();
    }

    /*!
     * Returns the first name of the current instance.
     *
     * \return A string.
     */
    const std::string& Person::getFirstName() const
    {
        return _firstName;
    }

    /*!
     * Returns the last name of the current instance.
     *
     * \return A string.
     */
    const std::string& Person::getLastName() const
    {
        return _lastName;
    }

    /*!
     * Returns the kind of event happening for the current
     * Person instance at a given date.
     *
     * Even if this person has several events in the same date,
     * the current method only returns the first found.
     *
     * \param date A reference to a Date instance.
     *
     * \return A DateKind code.
     */
    const DateKind Person::getEventForDate(const Date& date)
    {
        Dates& dates = getDates();
        int count = dates.size();
        DateKind event = NONE;
        for (int i = 0; i < count; i++)
        {
            Date current = dates.at(i);
            if (current.getMonth() == date.getMonth() && current.getDay() == date.getDay())
            {
                event = current.getKind();
                break;
            }
        }
        return event;
    }

    /*!
     * Returns all the Date instances associated to the current Person.
     * This method implements Fowler's "Lazy Load" pattern
     * (http://www.martinfowler.com/eaaCatalog/lazyLoad.html)
     * since it only loads the dates when the method is called.
     *
     * \return A vector of Date instances.
     */
    Dates& Person::getDates()
    {
        delete _dates;
        _dates = Date::findByPersonId(getId());
        return *_dates;
    }

    /*!
     * Adds the date of today as a special date, representing an
     * event, to the current instance.
     *
     * \param kind The type of event happening today.
     */
    void Person::addDate(const DateKind kind)
    {
        Date newDate;
        newDate.setKind(kind);
        addDate(newDate);
    }

    /*!
     * Adds the specified date as a special date, representing an
     * event, to the current instance.
     *
     * \param date A date in the form ISO 8601.
     * \param kind The type of event happening at the specified date.
     */
    void Person::addDate(const std::string& date, const DateKind kind)
    {
        Date newDate(date);
        newDate.setKind(kind);
        addDate(newDate);
    }

    /*!
     * Adds the specified date as a special date, representing an
     * event, to the current instance.
     *
     * \param year The year of the specified date.
     * \param month The month of the specified date.
     * \param day The day of the specified date.
     * \param kind The type of event happening at the specified date.
     */
    void Person::addDate(const int year, const int month, const int day, const DateKind kind)
    {
        Date newDate(year, month, day);
        newDate.setKind(kind);
        addDate(newDate);
    }

    /*!
     * Adds the specified date as a special date, representing an
     * event, to the current instance.
     *
     * \param date The date to add.
     */
    void Person::addDate(Date& date)
    {
        date.setPersonId(getId());
        if (isNew())
        {
            _dates->push_back(date);
        }
        else
        {
            date.save();
        }
        setDirty();
    }
}
