/*!
 * \file person.h
 * Contains the interface of the business::Person class.
 */

#ifndef PERSON_H_
#define PERSON_H_

#include <string>
using std::string;

#include <vector>
using std::vector;

#ifndef PERSISTABLE_H_
#include "persistable.h"
#endif

#ifndef DATE_H_
#include "date.h"
#endif

/*!
 * \namespace business
 * 
 * Contains all the core classes of the application. Classes on this
 * namespace only depend on the STL libraries, to make them highly 
 * portable and independent from the client application.
 */
namespace business
{
    // Forward declaration for the typedef below
    class Person;

    //! Represents a vector of Person instances; done for readability.
    typedef std::vector<Person> People;
    
    /*!
     * \class Person
     * Represents an individual, holding a first and a last name,
     * and possessing zero or more important dates, representing
     * important moments of her/his life.
     * 
     * This class is Persistable, and as such can be stored in the
     * "people" table of the SQLite database. 
     */
    class Person : public Persistable
    {
    public:

        //@{ \name Constructors and Destructor

        /*!
         * Public constructor. Creates an instance of the class
         * that does not exist (yet) in the database.
         * 
         * \param firstName A string with the first name.
         * \param lastName A string with the last name. 
         */
    	Person(const std::string& firstName,
                const std::string& lastName);

        /*!
         * Copy constructor.
         * 
         * \param rhs The instance to copy from.
         */
        Person(const Person& rhs);

        /*!
         * Virtual destructor.
         */
    	virtual ~Person();

        //@}
        
        //@{ \name Operators

        /*!
         * Assignment operator.
         * 
         * \param rhs The instance to copy from.
         * 
         * \return A reference to the current object.
         */
        Person& operator=(const Person& rhs);

        //@}

        //@{ \name Static Methods

        /*!
         * Static factory method that returns the instance of the
         * person class that corresponds to the ID passed in parameter.
         * Clients should assert or verify that the person has been
         * retrieved successfully before operating on it. 
         * 
         * \param id The ID of the person sought.
         * 
         * \return A pointer to a Person instance, or NULL.
         */
        static Person* findById(const PersistableId id);

        /*!
         * Static factory method that returns all the Person
         * instances that have some event at the specified date.
         * 
         * \param date The date of which we want people with events.
         * 
         * \return A vector of Person instances.
         */
        static People* findByDateEvent(const Date& date);
        
        /*!
         * Static factory method that returns all the Person
         * instances that exist in the database.
         * 
         * \return A vector of Person instances.
         */
        static People* findAll();

        /*!
         * Static method that performs the DELETE operation
         * on a single Person instance, specified by its ID.
         * 
         * \param id The ID of the instance to delete.
         */
        static void destroy(const PersistableId id);
        
        /*!
         * Static method that deletes all instances from the database.
         */
        static void destroyAll();

        //@}

        //@{ \name Setters and Getters

        /*!
         * Sets the first name of the current instance.
         * 
         * \param firstName A string.
         */
        void setFirstName(const std::string& firstName);

        /*!
         * Sets the last name of the current instance.
         * 
         * \param lastName A string.
         */
        void setLastName(const std::string& lastName);

        /*!
         * Returns the first name of the current instance.
         * 
         * \return A string.
         */
        const std::string& getFirstName() const;

        /*!
         * Returns the last name of the current instance.
         * 
         * \return A string.
         */
        const std::string& getLastName() const;

        /*!
         * Returns the kind of event happening for the current
         * Person instance at a given date.
         * 
         * Even if this person has several events in the same date,
         * the current method only returns the first found.
         * 
         * \param date A reference to a Date instance.
         * 
         * \return A DateKind code.
         */
        const DateKind getEventForDate(const Date& date);

        /*!
         * Returns all the Date instances associated to the current Person.
         * This method implements Fowler's "Lazy Load" pattern
         * (http://www.martinfowler.com/eaaCatalog/lazyLoad.html)
         * since it only loads the dates when the method is called.
         * 
         * \return A vector of Date instances.
         */
        Dates& getDates();
        
        /*!
         * Adds the date of today as a special date, representing an 
         * event, to the current instance.
         * 
         * \param kind The type of event happening today.
         */
        void addDate(const DateKind kind);

        /*!
         * Adds the specified date as a special date, representing an 
         * event, to the current instance.
         *
         * \param date A date in the form ISO 8601. 
         * \param kind The type of event happening at the specified date.
         */
        void addDate(const std::string& date, const DateKind kind);
        
        /*!
         * Adds the specified date as a special date, representing an 
         * event, to the current instance.
         * 
         * \param year The year of the specified date.
         * \param month The month of the specified date.
         * \param day The day of the specified date.
         * \param kind The type of event happening at the specified date.
         */
        void addDate(const int year, const int month, const int day, const DateKind kind);

        /*!
         * Adds the specified date as a special date, representing an 
         * event, to the current instance.
         * 
         * \param date The date to add.
         */
        void addDate(Date& date);

        //@}

    protected:

        /*!
         * Pure virtual method used by subclasses to store their
         * state in the database for the first time, using the
         * "INSERT" SQL statement. Subclasses must perform the insertion
         * and provide the ID that was given during the operation.
         * 
         * \return The ID of the current instance, as provided by SQLite.
         */
        virtual PersistableId insert();

        /*!
         * Pure virtual method used by subclasses to store their
         * state in the database after the first time, using the
         * "UPDATE" SQL statement.
         */
        virtual void update();

    private:

        /*!
         * Private constructor. Creates an instance of the class
         * that exists in the database. Only used by the "find()" factory
         * methods, to create instances using the data from the database. 
         * 
         * \param firstName A string with the first name.
         * \param lastName A string with the last name.
         * \param id The ID of the instance.
         */
        Person(const std::string& firstName,
                const std::string& lastName,
                const PersistableId id);

    private:

        //! The first name of the person represented by this instance.
        std::string        _firstName;
        
        //! Last name of the person represented by this instance.
        std::string        _lastName;
        
        //! The vector of dates associated to this instance.
        Dates* _dates;
    };
}

#endif /*PERSON_H_*/
