/*!
 * \file dataaccesslayer.h
 * Contains the interface of the data::DataAccessLayer class
 */

#ifndef DATAACCESSLAYER_H_
#define DATAACCESSLAYER_H_

#include <string>
using std::string;

#include <vector>
using std::vector;

#include <sqlite3.h>

/*!
 * \namespace data
 * This namespace holds the data access class, and constitutes the
 * "lowest" layer of the BirthdayCard application.
 */
namespace data
{
    /*!
     * \class DataAccessLayer
     * Proxy class used to access the SQLite database where all the information
     * about contacts and birthdays is stored. This class is a wrapper
     * around the SQLite API, adapted from 
     * http://souptonuts.sourceforge.net/code/simplesqlite3cpp2.cc.html
     * and it is also a Singleton, which clients can retrieve calling
     * the "get()" static method. 
     * 
     * For examples about how to use this class, please refer to the
     * test::DataAccessLayerTest unit test class.
     */
    class DataAccessLayer
    {
    public:

        /*!
         * Virtual destructor.
         */
        virtual ~DataAccessLayer();

        /*!
         * Sets the file name of the SQLite database. If no name is
         * given, the "BirthdayCard.db" name is used.
         * 
         * \param fileName A string with the file name.
         */
        static void setFileName(const std::string& fileName);
        
        /*!
         * Returns the singleton instance of this class.
         * 
         * \return A reference to the singleton instance.
         */
        static DataAccessLayer& get();

        /*!
         * Opens the connection to the SQLite database. Clients must
         * call this method before "executeQuery()" or they will get 
         * a runtime error.
         * 
         * \return A boolean value; true in case of success, false otherwise.
         */
        bool open();
        
        /*!
         * This is the heart of the class; this method takes a SQL query
         * in a std::string and sets the internal state of the class
         * to inform clients about the result of the query.
         * 
         * \param query The query to execute.
         * 
         * \return A boolean value; true in case of success, false otherwise.
         */
        bool executeQuery(const std::string& query);
        
        /*!
         * Closes the connection to the SQLite database. Clients must call
         * this method when they have finished using the database file.
         * 
         * The database also is closed automatically by the destructor.
         */
        void close();

        /*!
         * Returns the ID of the last element inserted or updated in the database.
         * 
         * \return A long long value (very long!)
         */
        const long long getLastRowId() const;

        /*!
         * Returns the last result code given by SQLite upon execution
         * of the last query.
         * 
         * \return An integer value.
         */
        const int getLastResultCode() const;

        /*!
         * Returns the last query executed by this instance,
         * regardless of it being successful or not.
         * 
         * \return A string.
         */
        const std::string& getLastQuery() const;

        /*!
         * Returns the last error message provided by SQLite.
         * 
         * \return A string.
         */
        const std::string& getLastErrorMsg() const;

        /*!
         * Returns a const reference to a vector, holding
         * the table headers corresponding to the last resultset
         * retrieved from the database.
         * 
         * \return A vector of string.
         */
        const std::vector<std::string>& getTableHeaders() const;

        /*!
         * Returns a const reference to a vector holding the data
         * of the last resultset retrieved from the database.
         * 
         * \return A vector of string.
         */
        const std::vector<std::string>& getData() const;

    private:

        /*!
         * Private constructor. Since this class is a singleton, 
         * only the static method "get()" is used to build a unique
         * instance for all the application. This singleton is not
         * thread-safe, but for this application, this is more than
         * enough.
         */
        DataAccessLayer();

        /*!
         * Private copy constructor.
         * 
         * This trick comes from "Effective C++, Third Edition"
         * by Scott Meyers (ISBN 0-321-33487-6)
         * "Item 6: Explicitly disallow the use of compiler-generated
         * functions you do not want", page 37
         * 
         * \param rhs The DataAccessLayer reference to copy from.
         */
        DataAccessLayer(const DataAccessLayer& rhs);
        
        /*!
         * Private assignment operator.
         * 
         * This trick comes from "Effective C++, Third Edition"
         * by Scott Meyers (ISBN 0-321-33487-6)
         * "Item 6: Explicitly disallow the use of compiler-generated
         * functions you do not want", page 37
         * 
         * \param rhs The DataAccessLayer reference to copy from.
         * 
         * \return A reference to the current DataAccessLayer.
         */
        DataAccessLayer& operator=(const DataAccessLayer& rhs);

    private:

        //! The file name of the SQLite database used by this class.
        static std::string _fileName;

    private:

        //! The result code of the last query executed on the database
        int _resultCode;
        
        //! The number of rows returned by the last query.
        int _numRows;
        
        //! The number of columns returned by the last query.
        int _numColumns;
        
        //! The ID of the last element inserted or updated in the database.
        long long _lastRowId;
        
        //! An error message returned by SQLite.
        std::string _errorMsg;
        
        //! The last query executed.
        std::string _lastQuery;
        
        //! A vector containing the headers of the last resultset.
        std::vector<std::string> _columnHeaders;
        
        //! A vector containing the data retrieved from the database.
        std::vector<std::string> _data;
        
        //! A variable representing the SQLite database itself.
        sqlite3* _db;
    };
}

#endif /*DATAACCESSLAYER_H_*/
