/*!
 * \file dataaccesslayertest.h
 * Contains the implementation of the test::DataAccessLayerTest class.
 */

#include "dataaccesslayertest.h"

#include <fstream> // for testing the existence of the database file

#include <string>
using std::string;

#include <stdio.h> // for the "remove" function

#include <vector>
using std::vector;

#include <sqlite3.h> // for the SQLite return codes

#ifndef DATAACCESSLAYER_H_
#include "../data/dataaccesslayer.h"
#endif

using data::DataAccessLayer;

/*!
 * \namespace tests
 * This namespace holds the classes that derive from 
 * CppUnit::TestFixture, containing unit tests for the BirthdayCard
 * application.
 */
namespace tests
{
    /*!
     * Constructor.
     */
    DataAccessLayerTest::DataAccessLayerTest()
    : _filename("test.db")
    {
    }
    
    /*!
     * Virtual destructor.
     */
    DataAccessLayerTest::~DataAccessLayerTest()
    {
    }
    
    /*!
     * Called by CppUnit before each test.
     */
    void DataAccessLayerTest::setUp()
    {
    }
    
    /*!
     * Called by CppUnit after each test.
     */
    void DataAccessLayerTest::tearDown()
    {
    }
    
    /*!
     * Tests the automatic creation of a database file.
     */
    void DataAccessLayerTest::testCreateDatabase()
    {
        // Delete the file if it already exists
        remove(_filename.c_str());
    
        // Calling the singleton "automagically" creates the file
        DataAccessLayer::setFileName(_filename);
        DataAccessLayer::get();
    
        // Check that the file exists
        std::fstream fin;
        fin.open(_filename.c_str(), std::fstream::in);
        CPPUNIT_ASSERT(fin.is_open());
        fin.close();
    }
    
    /*!
     * Tests the creation of tables.
     */
    void DataAccessLayerTest::testCreateTable()
    {
        DataAccessLayer& dal = DataAccessLayer::get();
        bool ok = dal.open();
        CPPUNIT_ASSERT(ok);
        ok = dal.executeQuery("CREATE TABLE test(test1 int, test2 varchar);");
        CPPUNIT_ASSERT(ok);
        dal.close();
    }
    
    /*!
     * Tests the insertion of values in the table created previously.
     */
    void DataAccessLayerTest::testInsertion()
    {
        DataAccessLayer& dal = DataAccessLayer::get();
        bool ok = true;
        for (int i = 0; i < 1000; i++)
        {
            ok = dal.open();
            CPPUNIT_ASSERT(ok);
            ok = dal.executeQuery("INSERT INTO test (test1, test2) VALUES (1, '2');");
            CPPUNIT_ASSERT(ok);
            dal.close();
        }
    }
    
    /*!
     * Tests the reading of values stored previously.
     */
    void DataAccessLayerTest::testSelect()
    {
        DataAccessLayer& dal = DataAccessLayer::get();
        bool ok = dal.open();
        CPPUNIT_ASSERT(ok);
        ok = dal.executeQuery("SELECT test1, test2 FROM test");
        CPPUNIT_ASSERT(ok);
        dal.close();
        
        const std::vector<std::string>& data = dal.getData();
        const size_t numberOfHeaders = dal.getTableHeaders().size();
        const size_t dataItems = data.size();
    
        CPPUNIT_ASSERT(dataItems == 2000);
        CPPUNIT_ASSERT(numberOfHeaders == 2);
        
        // Iterate over the data to see what's there
        for (size_t i = 0; i < dataItems / numberOfHeaders; i = i + numberOfHeaders)
        {
            CPPUNIT_ASSERT(data[i] == "1");
            CPPUNIT_ASSERT(data[i + 1] == "2");
        }
    }
}
