/*!
 * \file persontest.h
 * Contains the implementation of the test::PersonTest class.
 */

#include "persontest.h"

#include <string>
using std::string;

#include <sstream>
using std::stringstream;

#include <iostream>
using std::endl;

#include <vector>
using std::vector;

#ifndef DATAACCESSLAYER_H_
#include "../data/dataaccesslayer.h"
#endif
#ifndef PERSON_H_
#include "../business/person.h"
#endif
#ifndef DATE_H_
#include "../business/date.h"
#endif

using namespace business;
using namespace data;

/*!
 * \namespace tests
 * This namespace holds the classes that derive from
 * CppUnit::TestFixture, containing unit tests for the BirthdayCard
 * application.
 */
namespace tests
{
    /*!
     * Constructor.
     */
    PersonTest::PersonTest()
    : _personId  (1)
    , _firstName ("First")
    , _lastName  ("Last")
    , _filename  ("test.db")
    {
    }

    /*!
     * Virtual destructor.
     */
    PersonTest::~PersonTest()
    {
    }

    /*!
     * Called by CppUnit before each test.
     */
    void PersonTest::setUp()
    {
    }

    /*!
     * Called by CppUnit after each test.
     */
    void PersonTest::tearDown()
    {
    }

    /*!
     * Tests adding people to the database, including their
     * associated dates.
     */
    void PersonTest::testInsert()
    {
        DataAccessLayer::setFileName(_filename);
        DataAccessLayer& dal = DataAccessLayer::get();
        bool ok = dal.open();
        CPPUNIT_ASSERT(ok);
        ok = dal.executeQuery("CREATE TABLE people (id INTEGER PRIMARY KEY, first_name VARCHAR, last_name VARCHAR);");
        CPPUNIT_ASSERT(ok);
        ok = dal.executeQuery("CREATE TABLE dates (id INTEGER PRIMARY KEY, person_id INT, value DATE, kind INT);");
        CPPUNIT_ASSERT(ok);

        std::stringstream query;
        query << "CREATE TRIGGER delete_person BEFORE DELETE ON people";
        query << std::endl;
        query << "BEGIN";
        query << std::endl;
        query << "    DELETE FROM dates WHERE dates.person_id = old.id;";
        query << std::endl;
        query << "END;";
        ok = dal.executeQuery(query.str());
        CPPUNIT_ASSERT(ok);
        dal.close();

        Person person(_firstName, _lastName);
        CPPUNIT_ASSERT(person.isNew());
        CPPUNIT_ASSERT(person.isDirty());

        // Play with the date class as well, directly from the person
        // Here we add a date BEFORE the person is saved in database
        person.addDate(BIRTHDAY);
        person.save();
        CPPUNIT_ASSERT(!person.isNew());
        CPPUNIT_ASSERT(!person.isDirty());
        CPPUNIT_ASSERT_EQUAL(_personId, person.getId());

        // Here we add a date AFTER the person was saved
        CPPUNIT_ASSERT_EQUAL(1, (int)person.getDates().size());
        person.addDate(ANNIVERSARY);
        CPPUNIT_ASSERT_EQUAL(2, (int)person.getDates().size());
    }


    /*!
     * Tests getting people from the database, including their
     * associated dates.
     */
    void PersonTest::testSelect()
    {
        // Check that the person exists
        DataAccessLayer& dal = DataAccessLayer::get();
        bool ok = dal.open();
        CPPUNIT_ASSERT(ok);
        std::stringstream query;
        query << "SELECT id, first_name, last_name FROM people WHERE id = ";
        query << _personId;
        query << ";";
        ok = dal.executeQuery(query.str());
        CPPUNIT_ASSERT(ok);
        dal.close();

        const std::vector<std::string>& data = dal.getData();
        const size_t numberOfHeaders = dal.getTableHeaders().size();
        const size_t dataItems = data.size();

        CPPUNIT_ASSERT_EQUAL(size_t(3), dataItems);
        CPPUNIT_ASSERT_EQUAL(size_t(3), numberOfHeaders);

        Person* person = Person::findById(_personId);
        CPPUNIT_ASSERT(NULL != person);
        CPPUNIT_ASSERT_EQUAL(_personId, person->getId());
        CPPUNIT_ASSERT_EQUAL(_firstName, person->getFirstName());
        CPPUNIT_ASSERT_EQUAL(_lastName, person->getLastName());
        CPPUNIT_ASSERT(!person->isDirty());
        CPPUNIT_ASSERT(!person->isNew());
        CPPUNIT_ASSERT_EQUAL(2, (int)person->getDates().size());
        delete person;

        // Now get ALL the people in the database
        People* people = Person::findAll();
        CPPUNIT_ASSERT(NULL != people);
        CPPUNIT_ASSERT_EQUAL(size_t(1), people->size());

        person = &people->at(0);
        CPPUNIT_ASSERT_EQUAL(_firstName, person->getFirstName());
        CPPUNIT_ASSERT_EQUAL(_lastName, person->getLastName());
        CPPUNIT_ASSERT_EQUAL(_personId, person->getId());
        CPPUNIT_ASSERT(!person->isDirty());
        CPPUNIT_ASSERT(!person->isNew());
        CPPUNIT_ASSERT_EQUAL(2, (int)person->getDates().size());
        delete people;
    }


    /*!
     * Tests the copy constructor and the assignment operator.
     */
    void PersonTest::testCopy()
    {
        Person* person = Person::findById(_personId);
        CPPUNIT_ASSERT(NULL != person);
        CPPUNIT_ASSERT_EQUAL(_firstName, person->getFirstName());
        CPPUNIT_ASSERT_EQUAL(_lastName, person->getLastName());
        CPPUNIT_ASSERT_EQUAL(_personId, person->getId());
        CPPUNIT_ASSERT(!person->isDirty());
        CPPUNIT_ASSERT(!person->isNew());

        Person personCopy(*person);
        CPPUNIT_ASSERT_EQUAL(_firstName, personCopy.getFirstName());
        CPPUNIT_ASSERT_EQUAL(_lastName, personCopy.getLastName());
        CPPUNIT_ASSERT_EQUAL(_personId, personCopy.getId());
        CPPUNIT_ASSERT(!personCopy.isDirty());
        CPPUNIT_ASSERT(!personCopy.isNew());
        CPPUNIT_ASSERT_EQUAL(2, (int)personCopy.getDates().size());

        Person personAssigned = *person;
        CPPUNIT_ASSERT_EQUAL(_firstName, personAssigned.getFirstName());
        CPPUNIT_ASSERT_EQUAL(_lastName, personAssigned.getLastName());
        CPPUNIT_ASSERT_EQUAL(_personId, personAssigned.getId());
        CPPUNIT_ASSERT(!personAssigned.isDirty());
        CPPUNIT_ASSERT(!personAssigned.isNew());
        CPPUNIT_ASSERT_EQUAL(2, (int)personAssigned.getDates().size());

        delete person;
    }


    /*!
     * Tests updating people in the database.
     */
    void PersonTest::testUpdate()
    {
        _firstName = "John";
        _lastName = "Smith";

        Person* person = Person::findById(_personId);
        CPPUNIT_ASSERT(NULL != person);
        CPPUNIT_ASSERT_EQUAL(_personId, person->getId());
        CPPUNIT_ASSERT(!person->isNew());
        CPPUNIT_ASSERT(!person->isDirty());
        person->setFirstName(_firstName);
        person->setLastName(_lastName);
        CPPUNIT_ASSERT(person->isDirty());
        person->save();
        CPPUNIT_ASSERT(!person->isDirty());

        // Update a date of this person
        Date date = person->getDates().at(0);
        CPPUNIT_ASSERT(ANNIVERSARY == date.getKind());
        date.setKind(BIRTHDAY);
        date.save();
        CPPUNIT_ASSERT(BIRTHDAY == date.getKind());

        testSelect();
    }

    /*!
     * Tests removing people from the database.
     */
    void PersonTest::testDelete()
    {
        // Before deleting the person, delete one of its dates
        Person* person = Person::findById(_personId);
        Date date = person->getDates().at(1);
        int id = date.getId();
        Date::destroy(id);
        CPPUNIT_ASSERT_EQUAL(1, (int)person->getDates().size());
        delete person;

        Person::destroy(_personId);

        // Check that the person was deleted
        DataAccessLayer& dal = DataAccessLayer::get();
        bool ok = dal.open();
        CPPUNIT_ASSERT(ok);
        std::stringstream query;
        query << "SELECT id, first_name, last_name FROM people WHERE id = ";
        query << _personId;
        query << ";";
        ok = dal.executeQuery(query.str());
        CPPUNIT_ASSERT(ok);

        const std::vector<std::string>& data = dal.getData();
        const size_t numberOfHeaders = dal.getTableHeaders().size();
        const size_t dataItems = data.size();

        CPPUNIT_ASSERT_EQUAL(size_t(0), dataItems);
        CPPUNIT_ASSERT_EQUAL(size_t(0), numberOfHeaders);

        // Check that the dates associated to the person are gone as well
        // thanks to the TRIGGER in the SQLite database
        std::stringstream query2;
        query2 << "SELECT id, person_id, value, kind FROM dates WHERE person_id = ";
        query2 << _personId;
        query2 << ";";
        ok = dal.executeQuery(query2.str());
        CPPUNIT_ASSERT(ok);
        dal.close();

        const std::vector<std::string>& data2 = dal.getData();
        const size_t numberOfHeaders2 = dal.getTableHeaders().size();
        const size_t dataItems2 = data.size();

        CPPUNIT_ASSERT_EQUAL(size_t(0), dataItems2);
        CPPUNIT_ASSERT_EQUAL(size_t(0), numberOfHeaders2);

        person = Person::findById(_personId);
        CPPUNIT_ASSERT(NULL == person);
    }
}
