/*!
 * \file birthday.cpp
 * Contains the implementation of the ui::BirthdayCard class.
 * It also contains the major part of the Doxygen documentation
 * for the project.
 */

/*!
 * \mainpage BirthdayCard Application
 *
 * \author Adrian Kosmaczewski
 * \date October 17th, 2006
 *
 * BirthdayCard is a program that allows you to store important dates related
 * to people you know: for example their birthday, their wedding anniversary
 * or other important dates. It provides a list of people who has a "special
 * date" today (anniversary, birthday, etc), and also allows to create a
 * birthday or anniversary card.
 *
 * \section main_goals Goals
 *
 * The ideas behind BirthdayCard, ahead from the core functionality, are:
 *
 * \li Create a small GUI application, with a nice and reusable architecture;
 * \li Make it as portable as possible, in order for it to run seamlessly
 * in most common systems today;
 * \li BirthdayCard should be well-documented and easy to describe, and as such
 * the simplest usable design patterns will be used;
 * \li Make this application not only open source, but also public domain,
 * for others to learn from it, to use it and extend it, if needed.
 *
 * \section main_index Index
 *
 * More information about BirthdayCard can be found in the following pages:
 *
 * \li \subpage pageScreenshots "Screenshots"
 * \li \subpage pageInstallation "Installation"
 * \li \subpage pageArchitecture "Architecture"
 * \li \subpage pageDocumentation "Documentation"
 * \li \subpage pageEnhancement "Future Enhancements"
 * \li \subpage pageTools "External Tools"
 * \li \subpage pageReferences "References"
 *
 */
#include "birthdaycard.h"

#ifndef MAIN_H_
#include "main.h"
#endif

IMPLEMENT_APP(ui::BirthdayCard)

/*!
 * \namespace ui
 * Contains the classes that depend on the wxWidgets framework,
 * providing the GUI of the BirthdayCard application.
 */
namespace ui
{
    // 'Main program' equivalent: the program execution "starts" here
    bool BirthdayCard::OnInit()
    {
        Main* frame = new Main(_T("Birthday Card"));
        frame->Show(true);

        // success: wxApp::OnRun() will be called which will enter the main message
        // loop and the application will run. If we returned false here, the
        // application would exit immediately.
        return true;
    }

    int BirthdayCard::OnExit()
    {
      return 0;
    }
}

/*!
 * \page pageScreenshots Screenshots
 *
 * \section screen_tiger Mac OS X 10.4.8 "Tiger"
 *
 * \image html tiger.png
 *
 * \section screen_jaguar Mac OS X 10.2.8 "Jaguar"
 *
 * \image html jaguar.png
 *
 * \section screen_win2k Windows 2000 Professional SP4
 *
 * \image html windows.png
 *
 * \section screen_winxp Windows XP Professional SP2
 *
 * \image html windows_xp.png
 *
 * \section screen_gnome GNOME under Ubuntu Linux 6.06 "Dapper Drake"
 *
 * \image html gnome.png
 *
 * \section screen_xfce Xfce under Ubuntu Linux 6.06 "Dapper Drake"
 *
 * \image html xfce.png
 */

/*!
 * \page pageArchitecture Architecture
 *
 * BirthdayCard has a simple, layered architecture that sits on top
 * of a SQLite database file:
 *
 * \image html Architecture.png
 *
 * In the UI layer, the ui::Controller class provides the translation
 * needed between the standard-based business classes, and the
 * wxWidgets framework classes.
 *
 * \section arch_analysis Analysis Model
 *
 * \subsection arch_usecases Use Cases
 *
 * The following diagrams show the use cases of the BirthdayCard application:
 *
 * \image html uml/usecase1.png
 *
 * \image html uml/usecase2.png
 *
 * \subsection arch_usecases State Diagram
 *
 * The following diagram shows the different states of any Persistable object:
 *
 * \image html uml/personstate.png
 *
 * \subsection arch_design Design Model
 *
 * This is a UML diagram representing the Data and Business layers
 * of the Birthdaycard application:
 *
 * \image html uml/classes.png
 */

/*!
 * \page pageInstallation Installation
 *
 * \section install_win Windows
 *
 * Unzip the BirthdayCard-1.0-win-bin.zip file into its own folder,
 * and double-click the BirthdayCard.exe file. All the dependencies of the
 * application (particularly the sqlite3.dll file) are bundled in the
 * same ZIP file, and as such, the application folder can be placed
 * anywhere on your system.
 *
 * To uninstall, just drop the folder where you have unzipped the application
 * to the Recycle Bin.
 *
 * \section install_linux Linux
 *
 * Open a command window and type the following sequence of commands:
 *
 * \li cd ~
 * \li tar -xjvf BirthdayCard-1.0-linux-ppc-bin.tar.bz2
 * \li cd BirthdayCard-1.0-linux-ppc-bin
 * \li ./birthdaycard
 *
 * The Intel version of the binaries will be provided at a later stage.
 *
 * To uninstall:
 *
 * \li cd ~
 * \li rm -f BirthdayCard-1.0-linux-ppc-bin
 *
 * \section install_mac Mac OS X (10.2 to 104, Intel or PowerPC)
 *
 * Double-click on the BirthdayCard.dmg file; this will create a virtual disk
 * on your Desktop. In this virtual disk you will find the BirthdayCard application
 * bundle and the BirthdayCard.db database file in a folder named "BirthdayCard",
 * that you can place whenever you want (usually inside the Applications folder).
 *
 * Even if this application is not a Universal Binary, it should run on
 * Intel-based Macintoshes thanks to Rosetta (http://www.apple.com/rosetta/).
 *
 * To uninstall, just drop the "BirthdayCard" folder bundle to the Trash. The
 * application does not have any other dependencies.
 */

/*!
 * \page pageTools External Tools
 *
 * BirthdayCard was built using a myriad of external tools, most of them
 * available for free in the form of open-source projects. The idea behind
 * BirthdayCard is to be made of a C++ code as portable as possible.
 *
 * This page describes the tools used to create BirthdayCard.
 *
 * \li \ref tools_os
 * \li \ref tools_design
 * \li \ref tools_compilers
 * \li \ref tools_ides
 * \li \ref tools_testing
 * \li \ref tools_others
 *
 * \section tools_os Operating Systems
 *
 * BirthdayCard was almost completely built using Xcode on Mac OS X 10.4
 * "Tiger", but the application was successfully built and tested on the
 * following operating systems:
 *
 * \li Windows 2000 Professional Edition SP4
 * \li Windows XP Professional Edition SP2
 * \li Mac OS X 10.2.8 "Jaguar"
 * \li Mac OS X 10.4.7 "Tiger" (PowerPC)
 * \li Ubuntu Linux "Dapper Drake" 6.06 (PowerPC)
 *
 * \section tools_design Design Tools
 *
 * \li The UML diagrams included in the documentation were created using
 * the community edition of Visual Paradigm for UML 5.3
 * (http://www.visual-paradigm.com/product/vpuml/)
 * \li The architecture diagrams were created using OmniGraffle 3
 * (http://www.omnigroup.com/applications/omnigraffle/)
 *
 * \section tools_compilers Compilers
 *
 * The source code has been successfully built using the following compilers:
 *
 * \li gcc (GCC) 3.1 20020420 (prerelease) (Mac OS X 10.2.8 "Jaguar")
 * \li gcc version 4.0.1 (Apple Computer, Inc. build 5363) (Mac OS X 10.4.8 "Tiger")
 * \li gcc (GCC) 4.0.3 (Ubuntu 4.0.3-1ubuntu5) (Ubuntu Linux 6.06 "Dapper Drake")
 * \li Microsoft (R) 32-bit C/C++ Optimizing Compiler Version 13.10.3077 for 80x86
 * (Windows XP & 2000)
 *
 * \section tools_ides Integrated Development Environments and Build Tools
 *
 * The application can be opened using these IDEs:
 *
 * \li Apple Xcode 2.3 (Mac OS X 10.4 only): open the "BirthdayCard.xcodeproj" file.
 * \li Apple Project Builder (Mac OS X 10.2 only): open the "BirthdayCard.pbproj" file.
 * \li Visual Studio .NET 2003 (Windows only): open the "BirthdayCard.vcproj" file.
 * \li Eclipse 3.2 "Callisto" (http://www.eclipse.org/callisto/) with the CDT
 * (http://www.eclipse.org/cdt/)
 * (you can import the project from File/Import/General/Existing project into workspace)
 *
 * Note: even if Eclipse is cross-platform, currently the .project file that
 * is bundled with the source code can only be built from the Mac OS X "Tiger"
 * environment.
 *
 * Besides these IDEs, the application can be built from the command line
 * thanks to makefiles:
 *
 * \li GNU Make (http://www.gnu.org/software/make/)
 * \li Microsoft NMAKE utility
 * (http://msdn2.microsoft.com/en-us/library/dd9y37ha.aspx)
 *
 * \section tools_testing Open Source Tools
 *
 * The following open source, cross-platform tools were used to
 * create BirthdayCard and the current documentation:
 *
 * \li CppUnit 1.12 (http://cppunit.sourceforge.net/), directly downloadable
 * from http://prdownloads.sourceforge.net/cppunit/cppunit-1.12.0.tar.gz
 * \li SQLite 3.3.8 (http://www.sqlite.org/), directly downloadable from
 * http://www.sqlite.org/sqlite-3.3.8.tar.gz
 * \li wxWidgets 2.6.3 (http://www.wxwidgets.org/), directly downloadable from
 * http://prdownloads.sourceforge.net/wxwindows/wxWidgets-2.6.3.tar.bz2
 * \li Doxygen 1.4.7 (http://www.doxygen.org/)
 * \li Subversion 1.3 (http://subversion.tigris.org)
 *
 * \section tools_others Other Tools
 *
 * \li Microsoft HTML Help Workshop 3.0
 * (http://www.microsoft.com/downloads/details.aspx?familyid=00535334-c8a6-452f-9aa0-d597d16580cc&displaylang=en)
 * \li Crimson Editor (http://www.crimsoneditor.com/)
 * \li GnoCHM (http://gnochm.sourceforge.net/), CHM file viewer for the GNOME desktop
 * \li Chmox (http://chmox.sourceforge.net/), CHM file viewer for Mac OS X
 * \li The Windows ZIP files were generated with 7-zip (http://www.7-zip.org/)
 * \li Microsoft VBScript (http://msdn.microsoft.com/scripting) was used to
 * create the SQL file containing random dates.
 *
 */

/*!
 * \page pageEnhancement Future Enhancements
 *
 * For the next versions of BirthdayCard, the following enhancements could be done:
 *
 * \li Make the application more secure, sanitizing input in SQL queries to
 * avoid SQL injection problems.
 * \li Create a Universal Binary from the Xcode project (targeting the PowerPC
 * and the Intel processors at once).
 * \li Separate the business and data namespaces into a separate library (static
 * or dynamic).
 * \li Add logging and tracing, particularly for the SQL queries sent to the database.
 * \li Add scripting capabilities, making the application able to read from a file
 * a set of operations to execute.
 * \li Export contact information in the vCard
 * (http://www.imc.org/pdi/vcardoverview.html)
 * and vCalendar (http://www.imc.org/pdi/vcaloverview.html) formats.
 * \li Provide a Cocoa-based graphical front-end to the application using
 * Cocoa (http://developer.apple.com/cocoa/) and Objective-C++
 * (http://developer.apple.com/documentation/Cocoa/Conceptual/ObjectiveC/Articles/chapter_4_section_10.html)
 * \li Provide a KDevelop project (http://www.kdevelop.org/) for the Linux platform.
 * \li Create a base class for all the dialogs and frames in the GUI
 * of the application, inheriting from wxFrame and/or wxDialog.
 */

/*!
 * \page pageReferences References
 *
 * This page lists the external references of information used during
 * the development of the application.
 *
 * \section ref_books Books
 *
 * \li Date validation algorithm adapted from Deitel's "C++ How To Program,
 * Fifth Edition"
 * ISBN 0-13-185757-6, page 537
 * \li "Effective C++, Third Edition" by Scott Meyers (Addison Wesley,
 * ISBN 0-321-33487-6)
 * \li "C++ Cookbook", by Ryan Stephens, Christopher Diggins, Jonathan Turkanis and
 * Jeff Cogswell (O'Reilly, ISBN 0-596-00761-2)
 * \li "Cross-Platform GUI Programming with wxWidgets", by Julian Smart,
 * Kevin Hock and Stefan Csomor (Prentice Hall, ISBN 0-131-47381-6)
 *
 * \section ref_web References on the Web
 *
 * \li wxWidgets documentation (http://wxwidgets.org/docs/)
 * \li SQLite documentation (http://www.sqlite.org/docs.html)
 * \li CppUnit documentation (http://cppunit.sourceforge.net/doc/lastest/index.html)
 * \li Martin Fowler's Active Record Pattern is described in
 * http://www.martinfowler.com/eaaCatalog/activeRecord.html
 * \li Martin Fowler's Lazy Load Pattern is described in
 * http://www.martinfowler.com/eaaCatalog/lazyLoad.html
 * \li The list of random names used in the demo database is adapted from
 * http://www.xtra-rant.com/gennames/
 * \li The coding standards included in this document were generated using
 * Sven Rosvall's "Coding Standard Generator" (http://www.rosvall.ie/CSG/)
 * \li The SQLite C++ wrapper is adapted from
 * http://souptonuts.sourceforge.net/code/simplesqlite3cpp2.cc.html
 * \li The ISO 8601 standard ("YYYY-MM-DD") is described here
 * http://www.cl.cam.ac.uk/~mgk25/iso-time.html
 * \li Weekday calculation algorithm adapted from
 * http://en.wikipedia.org/wiki/Calculating_the_day_of_the_week
 * \li Basic date manipulation algorithms adapted from
 * http://www.informit.com/guides/content.asp?g=cplusplus&seqNum=65&rl=1
 * \li Optimized Singleton instance for single-threaded applications
 * taken from http://www.inquiry.com/techtips/cpp_pro/10min/10min0200.asp
 * \li Performance enhancement of SQLite 3 taken from
 * http://www.sqlite.org/pragma.html
 *
 */

/*!
 * \page pageDocumentation Documentation
 *
 * To generate the documentation, make sure that Doxygen 1.4.7 or later
 * (http://www.stack.nl/~dimitri/doxygen/) and dot (http://www.graphviz.org/) are
 * installed; type "doxygen" in the folder where the file "Doxyfile" is located.
 *
 * If you are using the Eclox (Eclipse plugin for Doxygen,
 * https://gna.org/projects/eclox) you can just right-click on the Doxyfile and
 * select "Build documentation" from within the Eclipse IDE. This will generate
 * HTML documentation under the doc/html folder.
 *
 * For Windows, the Doxyfile also assumes that the HTML Compiler (used to generate
 * this CHM file) is located in "C:\Program Files\HTML Help Workshop\hhc.exe".
 * CHM files can be viewed in the Macintosh using Chmox (http://chmox.sourceforge.net/)
 * and in Linux using GnoCHM (http://gnochm.sourceforge.net/)
 *
 */