/*!
 * \file controller.cpp
 * Contains the implementation of the ui::Controller class.
 */

#include "controller.h"

#include <iostream>
using std::cout;
using std::endl;

#ifndef PERSISTABLE_H_
#include "persistable.h"
#endif

#ifndef DATE_H_
#include "../business/date.h"
#endif

using namespace business;

#include <string>
using std::string;

#include <vector>
using std::vector;

/*!
 * \namespace ui
 * Contains the classes that depend on the wxWidgets framework,
 * providing the GUI of the BirthdayCard application.
 */
namespace ui
{
    Controller::Controller()
    : _people           (0)
    , _eventPeople      (0)
    , _currentPerson    (0)
    , _currentPersonId  (Persistable::DEFAULT_ID)
    , _currentDateId    (Persistable::DEFAULT_ID)
    , _currentFirstName (wxString())
    , _currentLastName  (wxString())
    , _statusBarText    (wxString())
    {
        delete _people;
        _people = Person::findAll();
    }

    Controller::~Controller()
    {
        delete _people;
        delete _currentPerson;
    }

    const wxString& Controller::getCurrentFirstName() const
    {
        return _currentFirstName;
    }

    const wxString& Controller::getCurrentLastName() const
    {
        return _currentLastName;
    }

    void Controller::fillPeopleList(wxListCtrl* list)
    {
        People::reverse_iterator peopleIterator;
        for(peopleIterator = _people->rbegin(); peopleIterator != _people->rend();
                peopleIterator++)
        {
            Person& currentPerson = *peopleIterator;
            wxString name;
            name << currentPerson.getFirstName().c_str();
            name << " ";
            name << currentPerson.getLastName().c_str();
            wxListItem item;
            long id = (long)currentPerson.getId();
            item.SetText(name);
            item.SetData(id);
            list->InsertItem(item);
        }

        // Update the status bar text
        size_t quantity = _people->size();
        _statusBarText = "";
        _statusBarText << quantity;
        if (quantity > 1)
        {
            _statusBarText << " people";
        }
        else
        {
            _statusBarText << " person";
        }
    }

    const wxString Controller::getNameForCard(long personId) const
    {
        wxString name;
        Person* person = Person::findById(personId);
        if (NULL != person)
        {
            name << person->getFirstName().c_str();
        }
        delete person;
        return name;
    }

    const wxString Controller::getTodayDateString() const
    {
        wxString dateString;
        Date today;
        dateString << today.getWeekday().c_str();
        dateString << ", ";
        dateString << today.getMonthName().c_str();
        dateString << " ";
        dateString << today.getDay();
        dateString << ", ";
        dateString << today.getYear();
        return dateString;
    }

    const wxString Controller::getEventForPersonId(long personId) const
    {
        wxString event;
        Person* person = Person::findById(personId);
        if (NULL == person)
        {
            event << "Event";
        }
        else
        {
            Date today;
            event << Date::getKindDescription(person->getEventForDate(today)).c_str();
        }
        delete person;
        return event;
    }
    
    void Controller::fillEventsList(wxListCtrl* list)
    {
        Date today;
        delete _eventPeople;
        _eventPeople = Person::findByDateEvent(today);
        int count = _eventPeople->size();
        for(int i = count - 1; i >= 0; i--)
        {
            Person& currentPerson = _eventPeople->at(i);
            wxString name;
            name << currentPerson.getFirstName().c_str();
            name << " ";
            name << currentPerson.getLastName().c_str();
            long personId = (long)currentPerson.getId();
            wxListItem item;
            item.SetColumn(0);
            item.SetText(name);
            item.SetData(personId);
            long index = list->InsertItem(item);
            wxString event;
            event << Date::getKindDescription(currentPerson.getEventForDate(today)).c_str();
            list->SetItem(index, 1, event);
        }
    }
    
    void Controller::updateCalendar(wxCalendarCtrl* calendar)
    {
        wxDateTime wxdate;
        if (Persistable::DEFAULT_ID == _currentDateId)
        {
            wxdate = wxDateTime::Now();
        }
        else
        {
            wxdate.Set(_currentDate->getDay(), wxDateTime::Month(_currentDate->getMonth() - 1), 
                _currentDate->getYear(), 12, 0, 0, 0);
        }
        calendar->SetDate(wxdate);
    }

    void Controller::updateDateKindControl(wxListBox* choice)
    {
        if (Persistable::DEFAULT_ID != _currentDateId)
        {
            choice->Select((int)_currentDate->getKind());
        }
    }
    
    void Controller::fillDatesList(wxListCtrl* list)
    {
        if (NULL != _currentPerson)
        {
            Dates& dates = _currentPerson->getDates();
            int count = dates.size();
            for(int i = 0; i < count; i++)
            {
                Date& current = dates.at(i);
                wxString date;
                date << current.toStandardString().c_str();
                long id = (long)current.getId();
                wxListItem item;
                item.SetText(date);
                item.SetData(id);
                item.SetColumn(0);
                long index = list->InsertItem(item);
                wxString event;
                event << Date::getKindDescription(current.getKind()).c_str();
                list->SetItem(index, 1, event);
            }
        }
    }
    
    const wxString& Controller::getStatusBarText() const
    {
        return _statusBarText;
    }
    
    void Controller::loadPerson(const int id)
    {
        _currentPersonId = id;
        _currentPerson = Person::findById(id);
        if (NULL != _currentPerson)
        {
            _currentFirstName = _currentPerson->getFirstName().c_str();
            _currentLastName = _currentPerson->getLastName().c_str();
        }
    }
    
    void Controller::loadDate(const int id)
    {
        _currentDate = Date::findById(id);
        if (NULL == _currentDate)
        {
            _currentDateId = Persistable::DEFAULT_ID;
        }
        else
        {
            _currentDateId = id;
        }
    }

    void Controller::savePerson(wxString& firstName, wxString& lastName)
    {
        // Convert from the wxWidgets framework classes to our own domain ones
        std::string first(firstName.ToAscii());
        std::string last(lastName.ToAscii());

        if (Persistable::DEFAULT_ID == _currentPersonId)
        {
            delete _currentPerson;
            _currentPerson = new Person(first, last);
            _currentPerson->save();
            _currentPersonId = _currentPerson->getId();
        }
        else
        {
            _currentPerson->setFirstName(first);
            _currentPerson->setLastName(last);
            _currentPerson->save();
        }
        delete _people;
        _people = Person::findAll();
    }

    void Controller::saveDate(wxDateTime& date, const int kind, bool editing)
    {
        if (editing)
        {
            Date::destroy(_currentDateId);
            delete _currentDate;
            _currentDate = new Date(date.GetYear(), (int)date.GetMonth() + 1, (int)date.GetDay());
            _currentDate->setKind((DateKind)kind);
            _currentDate->setPersonId(_currentPersonId);
            _currentDate->save();
            _currentDateId = _currentDate->getId();
        }
        else
        {
            Date newDate(date.GetYear(), (int)date.GetMonth() + 1, (int)date.GetDay());
            newDate.setKind((DateKind)kind);
            _currentPerson->addDate(newDate);
            _currentPerson->save();
        }
    }

    void Controller::deleteCurrentPerson()
    {
        if (Persistable::DEFAULT_ID != _currentPersonId)
        {
            Person::destroy(_currentPersonId);
            delete _people;
            _people = Person::findAll();
            _currentPerson = NULL;
            _currentPersonId = Persistable::DEFAULT_ID;
            _currentDate = NULL;
            _currentDateId = Persistable::DEFAULT_ID;
            _currentFirstName = "";
            _currentLastName = "";
        }
    }
    
    void Controller::deleteCurrentDate()
    {
        if (Persistable::DEFAULT_ID != _currentDateId)
        {
            Date::destroy(_currentDateId);
            _currentDateId = Persistable::DEFAULT_ID;
            delete _currentDate;
            _currentDate = NULL;
        }
    }
    
    void Controller::deleteAllPeople()
    {
        Person::destroyAll();
        delete _people;
        _people = Person::findAll();
        _currentPersonId = Persistable::DEFAULT_ID;
        _currentPerson = NULL;
        _currentFirstName = "";
        _currentLastName = "";
    }

    void Controller::prepareForNewPerson()
    {
        _currentPersonId = Persistable::DEFAULT_ID;
        _currentPerson = NULL;
        _currentFirstName = "";
        _currentLastName = "";
    }
    
    const bool Controller::isPersonSelected() const
    {
        bool personSelected = (Persistable::DEFAULT_ID != _currentPersonId);
        return personSelected;
    }
}
