/*!
 * \file controller.h
 * Contains the interface of the ui::Controller class.
 */

#ifndef CONTROLLER_H_
#define CONTROLLER_H_

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

#include <wx/datetime.h>
#include <wx/listctrl.h>
#include <wx/calctrl.h>

#ifndef PERSON_H_
#include "../business/person.h"
#endif

using namespace business;

/*!
 * \namespace ui
 * Contains the classes that depend on the wxWidgets framework,
 * providing the GUI of the BirthdayCard application.
 */
namespace ui
{
    /*!
     * \class Controller
     * The GUI for BirthdayCard has been created using the 
     * "Model-View-Controller" approach; this class fulfills, as
     * the name implies, the Controller part. It is the only class
     * of the whole GUI that knows about the classes in the
     * "business" namespace. It performs all the conversions
     * between the classes in that namespace (Person and Date)
     * and their siblings in the wxWidgets framework (particularly
     * the UI controls like wxListCtrl, or the wxDateTime class).
     * 
     * There is only one instance of this class, attached to the 
     * ui::Main class, and it is passed as reference to other
     * dialogs used throughout the application.
     */
    class Controller
    {
    public:
        Controller();
        virtual ~Controller();

        void loadPerson(const int id);
        void loadDate(const int id);
        const wxString& getCurrentFirstName() const;
        const wxString& getCurrentLastName() const;
        void fillPeopleList(wxListCtrl* list);
        void fillDatesList(wxListCtrl* list);
        const wxString& getStatusBarText() const;
        void savePerson(wxString& firstName, wxString& lastName);
        void deleteCurrentPerson();
        void deleteAllPeople();
        void deleteCurrentDate();
        void prepareForNewPerson();
        const bool isPersonSelected() const;
        const wxString getEventForPersonId(long personId) const;

        const wxString getTodayDateString() const;
        void fillEventsList(wxListCtrl* list);

        const wxString getNameForCard(long personId) const;

        void updateCalendar(wxCalendarCtrl* calendar);
        void updateDateKindControl(wxListBox* choice);
        void saveDate(wxDateTime& date, const int kind, bool editing);

    private:
        People*     _people;
        People*     _eventPeople;
        Person*     _currentPerson;
        Date*       _currentDate;
        int         _currentPersonId;
        int         _currentDateId;
        wxString    _currentFirstName;
        wxString    _currentLastName;
        wxString    _statusBarText;
    };
}

#endif /*CONTROLLER_H_*/
